"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask.ext.restplus import Resource, fields, marshal_with_field

from accountant import db

from .. import api

from ..models.accounts import Account
from ..models.operations import Operation

from .models import operation_model, operation_with_sold_model
from .parsers import account_range_parser
from .users import requires_auth

from ..fields import Object


ns = api.namespace('operation', description='Operation management')


@ns.route('/')
@api.doc(
    security='apikey',
    responses={
        401: 'Unauthorized'
    })
class OperationListResource(Resource):
    @requires_auth
    @api.response(200, 'OK', [operation_with_sold_model])
    @api.expect(parser=account_range_parser)
    @marshal_with_field(fields.List(Object(operation_with_sold_model)))
    def get(self):
        """
        Get operations with solds for a specific account.
        """
        data = account_range_parser.parse_args()

        return Operation.query(
            begin=data['begin'],
            end=data['end']
        ).filter(
            Operation.account_id == data['account_id']
        ).all(), 200

    @requires_auth
    @api.response(201, 'Operation created', operation_model)
    @api.response(404, 'Account not found')
    @api.response(406, 'Invalid operation data')
    @marshal_with_field(Object(operation_model))
    def post(self):
        """
        Create a new operation.
        """
        data = api.payload

        account_id = data['account_id']
        account = Account.query().get(account_id)

        if not account:
            api.abort(
                404,
                error_message='Account with id %d not found.' % account_id
            )

        # A new operation MUST NOT have an id;
        if 'id' in data and data['id']:
            api.abort(
                406,
                error_message='Id must not be provided on creation.'
            )

        operation = Operation(**data)

        db.session.add(operation)

        return operation, 201


@ns.route('/<int:id>')
@api.doc(
    security='apikey',
    params={
        'id': 'Id of the operation to manage'
    },
    responses={
        401: 'Unauthorized',
        404: 'Operation not found'
    })
class OperationResource(Resource):
    @requires_auth
    @api.response(200, 'OK', operation_model)
    @marshal_with_field(Object(operation_model))
    def get(self, id):
        """
        Get operation.
        """
        operation = db.session.query(Operation).get(id)

        if not operation:
            api.abort(
                404,
                error_message='Operation with id %d not found.' % id
            )

        return operation, 200

    @requires_auth
    @api.expect(operation_model)
    @api.response(200, 'OK', operation_model)
    @api.response(406, 'Invalid operation data')
    @marshal_with_field(Object(operation_model))
    def post(self, id):
        data = api.payload

        # Check ID consistency.
        if 'id' in data and data['id'] and data['id'] != id:
            api.abort(
                406,
                error_message='Id must not be provided or changed on update.'
            )

        operation = db.session.query(Operation).get(id)

        if not operation:
            api.abort(
                404,
                error_message='Operation with id %d not found.' % id
            )

        # FIXME check account_id consistency.

        # SQLAlchemy objects ignore __dict__.update() with merge.
        for k, v in data.items():
            setattr(operation, k, v)

        db.session.merge(operation)

        return operation, 200

    @requires_auth
    @api.response(204, 'Operation deleted', operation_model)
    @marshal_with_field(Object(operation_model))
    def delete(self, id):
        operation = db.session.query(Operation).get(id)

        if not operation:
            api.abort(
                404,
                error_message='Operation with id %d not found.' % id
            )

        db.session.delete(operation)

        return None, 204
