"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask.ext.restplus import Resource, fields, marshal_with_field

from sqlalchemy import true

from accountant import db

from .. import api

from ..models.accounts import Account
from ..models.operations import Operation
from ..models.scheduled_operations import ScheduledOperation

from .models import scheduled_operation_model
from .parsers import account_id_parser
from .users import requires_auth

from ..fields import Object


ns = api.namespace(
    'scheduled_operation',
    description='Scheduled operation management'
)


@ns.route('/')
@api.doc(
    security='apikey',
    responses={
        401: 'Unauthorized',
    })
class ScheduledOperationListResource(Resource):
    @requires_auth
    @api.expect(account_id_parser)
    @api.response(200, 'OK', [scheduled_operation_model])
    @marshal_with_field(fields.List(Object(scheduled_operation_model)))
    def get(self):
        """
        Get all scheduled operation for an account.
        """
        data = account_id_parser.parse_args()

        return ScheduledOperation.query().filter_by(**data).all(), 200

    @requires_auth
    @api.expect(scheduled_operation_model)
    @api.response(200, 'OK', scheduled_operation_model)
    @api.response(404, 'Account not found')
    @api.response(406, 'Invalid operation data')
    @marshal_with_field(Object(scheduled_operation_model))
    def post(self):
        """
        Add a new scheduled operation.
        """
        data = api.payload

        account_id = data['account_id']
        account = Account.query().get(account_id)

        if not account:
            api.abort(
                404,
                error_message='Account with id %d not found.' % account_id
            )

        # A new scheduled operation MUST NOT have an id;
        if 'id' in data and data['id']:
            api.abort(
                406,
                error_message='Id must not be provided on creation.'
            )

        scheduled_operation = ScheduledOperation(**data)

        db.session.add(scheduled_operation)

        scheduled_operation.reschedule()

        db.session.flush()

        return scheduled_operation, 201


@ns.route('/<int:id>')
@api.doc(
    security='apikey',
    params={
        'id': 'Id of the scheduled operation to manage'
    },
    responses={
        401: 'Unauthorized',
        404: 'Scheduled operation not found'
    })
class ScheduledOperationResource(Resource):
    @requires_auth
    @api.response(200, 'OK', scheduled_operation_model)
    @marshal_with_field(Object(scheduled_operation_model))
    def get(self, id):
        """
        Get scheduled operation.
        """
        scheduled_operation = ScheduledOperation.query().get(id)

        if not scheduled_operation:
            api.abort(
                404,
                error_message='Scheduled operation with id %d not found.' % id
            )

        return scheduled_operation, 200

    @requires_auth
    @api.response(200, 'OK', scheduled_operation_model)
    @api.response(406, 'Invalid scheduled operation data')
    @api.expect(scheduled_operation_model)
    @marshal_with_field(Object(scheduled_operation_model))
    def post(self, id):
        """
        Update a scheduled operation.
        """
        data = api.payload

        # Check ID consistency.
        if 'id' in data and data['id'] and data['id'] != id:
            api.abort(
                406,
                error_message='Id must not be provided or changed on update.'
            )

        scheduled_operation = ScheduledOperation.query().get(id)

        if not scheduled_operation:
            api.abort(
                404,
                error_message='Scheduled operation with id %d not found.' % id
            )

        # FIXME check account_id consistency.

        # SQLAlchemy objects ignore __dict__.update() with merge.
        for k, v in data.items():
            setattr(scheduled_operation, k, v)

        db.session.merge(scheduled_operation)

        scheduled_operation.reschedule()

        db.session.flush()

        return scheduled_operation, 200

    @requires_auth
    @api.response(200, 'OK', scheduled_operation_model)
    @api.response(409, 'Cannot be deleted')
    @marshal_with_field(Object(scheduled_operation_model))
    def delete(self, id):
        """
        Delete a scheduled operation.
        """
        scheduled_operation = ScheduledOperation.query().get(id)

        if not scheduled_operation:
            api.abort(
                404,
                error_message='Scheduled operation with id %d not found.' % id
            )

        operations = scheduled_operation.operations.filter(
            Operation.confirmed == true()
        ).count()

        if operations:
            api.abort(
                409,
                error_message='There are still confirmed operations \
                associated to this scheduled operation.')

        # Delete unconfirmed operations
        scheduled_operation.operations.delete()

        db.session.delete(scheduled_operation)

        return None, 204
